#pragma once

#include <afxcoll.h>
#include <afxtempl.h>

#include <string>
#include <map>
#include <vector>
#include <hash_map>

class CPassCounter
{
private:
	bool	m_bOperating;	// true:쒆 false:~
	DWORD	m_nResetCount;	// JE^ZbgCount
	DWORD	m_nStopCount;	// ~Count
public:
	CPassCounter()
		{
			m_bOperating = false;
			ResetCount();
		}
	bool IsOperating()				{return m_bOperating;}
	void Start()
		{
			if( m_bOperating ) return;
			m_nResetCount += ::GetTickCount()-m_nStopCount;
			m_bOperating = true;
		}
	void Stop()
		{
			if( !m_bOperating ) return;
			m_nStopCount = ::GetTickCount();
			m_bOperating = false;
		}
	void ResetCount()				{SetCount(0);}
	void SetCount(DWORD nCount)
		{
			m_nResetCount = ::GetTickCount() - nCount;
			m_nStopCount = m_nResetCount + nCount;
		}
	DWORD GetCount()const
		{
			if(m_bOperating) return ::GetTickCount()-m_nResetCount;
			return m_nStopCount-m_nResetCount;
		}
};

class CMapBase{
public:
	CMapBase()
		{
		}
	virtual ~CMapBase(){};
	typedef int (CMapBase::*BENCHFUNC)(const std::vector<std::string>&);
	int BenchInsert(const std::vector<std::string>& arykey);
	int BenchFind(const std::vector<std::string>& arykey);
public:
	virtual CString GetName()=0;
protected:
	virtual void OnInsert(const std::string& sKey,int nValue)=0;
	virtual bool OnFind(const std::string& sKey,int& nValue)=0;
	virtual void OnClear(){}
};

class CMap_map : public CMapBase
{
	std::map<std::string,int>	m_map;
protected:
	virtual void OnInsert(const std::string& sKey,int nValue)
		{
			m_map.insert( std::pair<std::string,int>(sKey,nValue) );
		}

	virtual bool OnFind(const std::string& sKey,int& nValue)
		{
			std::map<std::string,int>::const_iterator iter = m_map.find(sKey);
			if( iter == m_map.end() ) return false;
			nValue = iter->second;
			return true;
		}

	virtual void OnClear()
		{
			m_map.clear();
		}

	virtual CString GetName()
		{
			return _T("STL:map");
		}
};

class CMap_hash_map : public CMapBase
{
	stdext::hash_map<std::string,int>	m_map;
protected:
	virtual void OnInsert(const std::string& sKey,int nValue)
		{
			m_map.insert( std::pair<std::string,int>(sKey,nValue) );
		}

	virtual bool OnFind(const std::string& sKey,int& nValue)
		{
			stdext::hash_map<std::string,int>::const_iterator iter = m_map.find(sKey);
			if( iter == m_map.end() ) return false;
			nValue = iter->second;
			return true;
		}

	virtual void OnClear()
		{
			m_map.clear();
		}

	virtual CString GetName()
		{
			return _T("STL:hash_map");
		}
};

class CMap_CMap : public CMapBase
{
protected:
	CMap<CString,LPCTSTR,int,int&>	m_map;
protected:
	virtual void OnInsert(const std::string& sKey,int nValue)
		{
			m_map.SetAt( sKey.c_str(), nValue );
		}

	virtual bool OnFind(const std::string& sKey,int& nValue)
		{
			return m_map.Lookup(sKey.c_str(),nValue) != false;
		}

	virtual void OnClear()
		{
			m_map.RemoveAll();
		}

	virtual CString GetName()
		{
			return _T("MFC:CMap");
		}

};

class CMap_CMap1009 : public CMap_CMap
{
public:
	CMap_CMap1009()
		{
			m_map.InitHashTable(1009);
		}

	virtual CString GetName()
		{
			return _T("MFC:CMap 1009");
		}
};
